#include <pwd.h>
#include <time.h>
#include <errno.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>

#include "mic.h"
#include "config.h"
#include "vyroci.h"

#define	debug	dummy

static int OneOf(const char *str, ...)
{
	va_list	args;
	char		*val;
	va_start(args,str);
	
	while((val=va_arg(args,char*))!=NULL)
		if(strcasecmp(str,val)==0)
			return 1;
	return 0;
}

static void InitFlags(FILE *f)
{
	char	buf[1000];
	
	debug("  *** Flags:\n");
	
	/* --- NDflag --- */
	debug("\tNDflag  = ");
	if(parseconf(f,"vyroci","NDflag",buf)==0)
		debug("... not found. Using default: %s\n",(cfg.ndflag=NDFLAG_VAL?1:0)?"On":"Off");
	else
		if(OneOf(buf,"1","Yes","Y","True","T","On",NULL))
		{
			debug("%s ... On\n",buf);
			cfg.ndflag=1;
		}
		else if(OneOf(buf,"0","No","N","False","F","Off",NULL))
		{
			debug("%s ... Off\n",buf);
			cfg.ndflag=0;
		}
		else
			debug("%s ... unknown value. Using default: %s\n",buf,(cfg.ndflag=NDFLAG_VAL?1:0)?"On":"Off");
	
	/* --- NDdays --- */
	if(cfg.ndflag)
	{
		debug("\tNDdays  = ");
		if(parseconf(f,"vyroci","NDdays",buf)==0)
			debug("... nof found. Using default: %d\n",cfg.nddays=NDDAYS_VAL);
		else if(sscanf(buf,"%d",&cfg.nddays)==1)
			debug("%d ... OK\n",cfg.nddays);
		else
			debug("%s ... not recognized. Using default: %d\n",buf,cfg.nddays=NDDAYS_VAL);
	}
	
	/* --- ANflag --- */
	debug("\tANflag  = ");
	if(parseconf(f,"vyroci","ANflag",buf)==0)
		debug("... not found. Using default: %s\n",(cfg.anflag=ANFLAG_VAL?1:0)?"On":"Off");
	else
		if(OneOf(buf,"1","Yes","Y","True","T","On",NULL))
		{
			debug("%s ... On\n",buf);
			cfg.anflag=1;
		}
		else if(OneOf(buf,"0","No","N","False","F","Off",NULL))
		{
			debug("%s ... Off\n",buf);
			cfg.anflag=0;
		}
		else
			debug("%s ... unknown value. Using default: %s\n",buf,(cfg.anflag=ANFLAG_VAL?1:0)?"On":"Off");
	
	/* --- ANdays --- */
	if(cfg.anflag)
	{
		debug("\tANdays  = ");
		if(parseconf(f,"vyroci","ANdays",buf)==0)
			debug("... nof found. Using default: %d\n",cfg.andays=ANDAYS_VAL);
		else if(sscanf(buf,"%d",&cfg.andays)==1)
			debug("%d ... OK\n",cfg.andays);
		else
			debug("%s ... not recognized. Using default: %d\n",buf,cfg.andays=ANDAYS_VAL);
	}
	
	/* --- OTflag --- */
	debug("\tOTflag  = ");
	if(parseconf(f,"vyroci","OTflag",buf)==0)
		debug("... not found. Using default: %s\n",(cfg.otflag=OTFLAG_VAL?1:0)?"On":"Off");
	else
		if(OneOf(buf,"1","Yes","Y","True","T","On",NULL))
		{
			debug("%s ... On\n",buf);
			cfg.otflag=1;
		}
		else if(OneOf(buf,"0","No","N","False","F","Off",NULL))
		{
			debug("%s ... Off\n",buf);
			cfg.otflag=0;
		}
		else
			debug("%s ... unknown value. Using default: %s\n",buf,(cfg.otflag=OTFLAG_VAL?1:0)?"On":"Off");
	
	/* --- OTdays --- */
	if(cfg.otflag)
	{
		debug("\tOTdays  = ");
		if(parseconf(f,"vyroci","OTdays",buf)==0)
			debug("... nof found. Using default: %d\n",cfg.otdays=OTDAYS_VAL);
		else if(sscanf(buf,"%d",&cfg.otdays)==1)
			debug("%d ... OK\n",cfg.otdays);
		else
			debug("%s ... not recognized. Using default: %d\n",buf,cfg.otdays=OTDAYS_VAL);
	}
	
}

static void InitFiles(void)
{
	FILE	*f;
	char	buf[1000];
	
	sprintf(cfg.pdir,"%s/%s",getpwuid(getuid())->pw_dir,SUBDIR_NM);
	sprintf(cfg.pconfig,"%s/%s",cfg.pdir,CONFIG_NM);
	if((f=fopen(cfg.pconfig,"r"))==NULL)
	{
		fprintf(stderr,"Nemuzu otevrit soubor '%s' (%s). %c\n",cfg.pconfig,errstr(errno),7);
		fprintf(stdout,"Prikazem 'vyroci I' ho muzes vytvorit. (samozrejme ti s nim pomuzu)\n");
		exit(0);
	}
	else
	{
		debug("Processing file %s:\n",cfg.pconfig);
		debug("\tVersion = ");
		if(parseconf(f,"vyroci","Version",buf)==0)
		{
			debug("... not found. Ignoring file!\n");
			failprog(1,"Konfiguracni soubor ma nejaky mouchy. Nemuzu najit radek \"Version = ...\"%c\n",7);
		}
		else
			if(atoi(buf)!=1)
			{
				debug("%s ... invalid value ('1' expected)\n",buf);
				failprog(1,"Nejak tomu konfiguracnimu souboru nerozumim.\n'Version' ma byt 1, ale je %s. Jdu pryc!%c\n",buf,7);
			}
			else
			{
				debug("%s ... OK\n",buf);
				
				debug("\tOnceAday = ");
				if(parseconf(f,"vyroci","OnceAday",buf)==0)
				{
					sprintf(cfg.odfile,"%s/%s/%s",getpwuid(getuid())->pw_dir,SUBDIR_NM,ODFILE_NM);
					debug("... not found. Using %s\n",cfg.odfile);
				}
				else
				{
					if(buf[0]=='/')
						strcpy(cfg.odfile,buf);
					else
						sprintf(cfg.odfile,"%s/%s",cfg.pdir,buf);
					debug("%s ... OK\n",cfg.odfile);
				}
				
				debug("\tAnnual  = ");
				if(parseconf(f,"vyroci","Annual",buf)==0)
				{
					sprintf(cfg.pannual,"%s/%s/%s",getpwuid(getuid())->pw_dir,SUBDIR_NM,ANNUAL_NM);
					debug("... not found. Using %s\n",cfg.pannual);
				}
				else
				{
					if(buf[0]=='/')
						strcpy(cfg.pannual,buf);
					else
						sprintf(cfg.pannual,"%s/%s",cfg.pdir,buf);
					debug("%s ... OK\n",cfg.pannual);
				}
				
				debug("\tNameday = ");
				if(parseconf(f,"vyroci","Nameday",buf)==0)
				{
					sprintf(cfg.pnameday,"%s/%s/%s",getpwuid(getuid())->pw_dir,SUBDIR_NM,NAMEDAY_NM);
					debug("... not found. Using %s\n",cfg.pnameday);
				}
				else
				{
					if(buf[0]=='/')
						strcpy(cfg.pnameday,buf);
					else
						sprintf(cfg.pnameday,"%s/%s",cfg.pdir,buf);
					debug("%s ... OK\n",cfg.pnameday);
				}
				
				debug("\tOnetime = ");
				if(parseconf(f,"vyroci","Onetime",buf)==0)
				{
					sprintf(cfg.ponetime,"%s/%s/%s",getpwuid(getuid())->pw_dir,SUBDIR_NM,ONETIME_NM);
					debug("... not found. Using %s\n",cfg.ponetime);
				}
				else
				{
					if(buf[0]=='/')
						strcpy(cfg.ponetime,buf);
					else
						sprintf(cfg.ponetime,"%s/%s",cfg.pdir,buf);
						
					debug("%s ... OK\n",cfg.ponetime);
				}
				InitFlags(f);
			}
	}
	fclose(f);
}

static void InitTime(void)
{
	struct tm	tm;
	time_t		ti;
	
	time(&ti);
	bcopy(localtime(&ti),&tm,sizeof(struct tm));
	cfg.tdday=tm.tm_mday;
	cfg.tdmon=tm.tm_mon+1;
	cfg.tdyear=tm.tm_year+1900;
	cfg.tdabs=AbsDay(cfg.tdyear,cfg.tdmon,cfg.tdday);
}

void InitConfig(void)
{
	InitTime();
	InitFiles();
}

